 #
 #   $Id: Tiki.pm,v 1.7 2004/05/01 14:23:42 zarahg Exp $
 #
 #   This  file  is  part of the  PodWiki  web-based authoring tool.
 #
 #   By  accessing  this software,  PodWiki,  you are  duly informed
 #   of and  agree to be  bound  by the  conditions  described below
 #   in this notice:
 #
 #   This software product, PodWiki,  is developed by  Thomas Linden
 #   and   copyrighted  (C) 2003-2004   by  Thomas Linden,  with all
 #   rights reserved.
 #
 #   There is  no charge for  PodWiki software. You can redistribute
 #   it and/or modify it under  the terms of the  GNU General Public
 #   License, which is incorporated by reference herein.
 #
 #   PodWiki is distributed WITHOUT ANY WARRANTY,IMPLIED OR EXPRESS,
 #   OF  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that
 #   the  use of it will not infringe on any third party's intellec-
 #   tual property rights.
 #
 #   You  should  have  received a  copy  of  the GNU General Public
 #   License along with PodWiki.  Copies  can also be obtained from:
 #
 #     http://www.gnu.org/copyleft/gpl.html
 #
 #   or by writing to:
 #
 #     Free Software Foundation, Inc.
 #     59 Temple Place, Suite 330
 #     Boston, MA 02111-1307
 #     USA
 #
 #   Or contact:
 #
 #    "Thomas Linden" <tom@daemon.de>
 #
 #
 #   Additional Copyrights:
 #
 #    Tiki.pm - Copyright 2004 by David Bright
 #

package PodWiki::Formatter::Tiki;

$PodWiki::Formatter::Tiki::VERSION = "2.01";

use lib qw(. lib);

use strict;
#no strict "refs";
use warnings;

use Exporter;

use Data::Dumper;
use Pod::Tree;
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use CGI::Cookie;

use PodWiki::Config;
use PodWiki::Vars;

use vars qw(@ISA @EXPORT @EXPORT_OK @EXPORT_TAGS @docpages $config);


@ISA=qw(Exporter PodWiki::Formatter);
@EXPORT=qw();
@EXPORT_OK=qw();
@EXPORT_TAGS=();

sub new {
  my($this, %param) = @_;
  my $class = ref($this) || $this;
  my $self = \%param;
  $config = $self->{config};
  $config->fetch();
  bless($self, $class);
  return $self;
}

sub tags {
  my($this) = @_;
  return qw(tiki);
}

sub do_img {
  my($this, $params) = @_;
  my($link, @params, %params);

  local $_ = $params;

  @params = split; # Split on whitespace to separate keyword=value pairs
  foreach my $param (@params) {
	my($key, $val) = split(/=/, $param);
	$key =~ s/^\s*(\S*)"\s*$/$1/; # Remove any leading and trailing whitespace
	$val =~ s/^\s*"([^"]*)"\s*$/$1/; # Remove any double quotes around value, plus any leading and trailing whitespace
	$params{$key} = $val;
  }

  $link = "";
  if (defined($params{"src"})) {
	$link = "<img src=\"$params{src}\"";
	delete($params{"src"}); # Delete the src keyword so we don't process it again below
  } else {
	$link = "<img src=\"\""; # Missing src keyword
  }
	
  foreach my $key (keys(%params)) {
	$link .= "," . $key . '="' . $params{$key} . '"'
  } 
  $link .= ">";
  return $link;
}

sub format {
  my($this, $text) = @_;
  local $_ = $text;

  $_ ="\n\n$_\n";

  # Scarf out preformatted sections and replace with a unique key
  my @pp = ();
  s/\~pp\~((.|\n)*?)\~\/pp\~/"~pp~" . (push(@pp, $1), $#pp) . "~\/pp~"/ge;
  # Scarf out non-parsed sections and replace with a unique key
  my @np = ();
  s/\~np\~((.|\n)*?)\~\/np\~/"~np~" . (push(@np, $1), $#np) . "~\/np~"/ge;


  # heads:
  # !Title
  my %h = ( '!' => 1, '!!' => 2, '!!!' => 3, '!!!!' => 4, '!!!!!' => 5, '!!!!!!' => 6);
  s/(?<=\n)(!)+[-+]?(.*)\s*\n/\n<h$h{$1}>$2<\/h$h{$1}>\n/g;

  # formats:
  # __bold__
  # ''italic''
  s/(?<!\\)__(.*?)__/<b>$1<\/b>/g;           # bold
  s/(?<![<\/])''(.*?)(?<!<)''/<i>$1<\/i>/g;  # italic

  # image var post-processing
  # $ok  (maintained by PodWiki::Vars) => {picture file=ok.png}
  s/G<([^>]*)>/{picture file=$1}/g;

  # images:
  # image.png
  #s/([\w\-]*\.png)/<img src="$config->{gfx}\/$1" alt="$1">/g;
  s/\{picture file=([^\}]+)\}/<img src="$config->{gfx}\/$1" alt="$1">/g;
  s/(\{img\s+([^\}]+)})/$this->do_img($2)/ge;

 

  # Replace boxes
  s/\^([^\^]+)\^/<div class="simplebox">$1<\/div>/g;
  # Replace colors ~~color:text~~
  s/\~\~([^\:]+):([^\~]+)\~\~/<span style="color:$1;">$2<\/span/g;
  # Underlined text
  s/===([^\=]+)===/<span style="text-decoration:underline;">$1<\/span>/g;
  # Center text
  s/::([^\:]+)::/<div align="center">$1<\/div>/g;
  # Monospace text
  s/-\+(.*?)\+-/<tt>$1<\/tt>/g;

  # Titlebars
  s/(^|\n)-=(.+?)=-/$1<div class='titlebar'>$2<\/div>\n/;
  # Replace definition lists
  s/(^|\n);([^:]+):([^\n]+)/$1<dl><dt>$2<\/dt><dd>$3<\/dd><\/dl>/;


  my $page_regex = '[A-Za-z0-9_\x80-\xFF][\.: A-Za-z0-9_\-\x80-\xFF]*[A-Za-z0-9_\x80-\xFF]';
  # CamelCase links
  #page_regex = '[A-Z][a-z0-9_\-]+[A-Z][a-z0-9_\-]+[A-Za-z0-9\-_]*';
  s/([ \n\t\r\,\;]|^)([A-Z][a-z0-9_\-]+[A-Z][a-z0-9_\-]+[A-Za-z0-9\-_]*)($|[ \n\t\r\,\;\.])/$1 . $this->pod_link($2) . $3/ge;
  # Now, get rid of the )) (( around CamelCase non-links
  s/([ \n\t\r\,\;>]|^)\)\)([^\(]+)\(\(($|[ \n\t\r\,\;\.<])/$1$2$3/g;

  # Free links - reverse the order of the target and description
  s/\(\(($page_regex)\|([^\)]+)\)\)/$this->pod_link($2 . "|" . $1)/ge;
  # Link with no description
  s/\(\(($page_regex)\)\)/$this->pod_link($1)/ge;

  # External links - reverse the order of the target and description
  s/([^\[])\[([^\[\]|]+)\|([^\]]*)\]/$1<img src="$config->{gfx}\/elink.png"> <a href="$2">$3<\/a>/gi;
  # Link with no description
  s/([^\[])\[([^\[\]]+)\]/$1<img src="$config->{gfx}\/elink.png"> <a href="$2">$2<\/a>/gi;
  # Bracketed text
  s/\[\[([^\]]*)\]/[$1]/gi;

  # include tag, prepare for split (see below)
  # We have to throw away the start/stop markers ($3, $5 in regexp below) because that is not implemented by POD
  s/\{INCLUDE\s*\(\s*page\s*=>\s*"?([^,\)]*)"?\s*(,\s*start\s*=>([^,\)]*)){0,1}(,\s*stop\s*=>(^[^,\)]*)){0,1}\)\}\s*\{INCLUDE\}/\n\nP<$1>\n\n/g;
  # Horizontal lines
  s/(^|\n)---.*?($|\n)/$1<hr \/>$2/g;
		
  # resolve entities
  s/E<([^>]*)>/&$1;/g;


  # Lists
  {
      my $curlistlevel = 0;
      my $newlistlevel = 0;
      my @closelist = ();
      my $newdata;
      my $listart;
      my $liend;
      my $liitem;
  
    foreach my $node (split /\n/) {
	  my $line = $node;
	  my $litype;

	  $listart = "";	# Stuff to put at the beginning of the line (blank or <li>)
	  $liend = "";		# Stuff to put at the end of the line (blank or </li>)
	  $newlistlevel = 0;
	  if ($line =~ m/^((\*|\#)+)(.*)$/) {
	      $litype = $1;
	      $liitem = $3;
	      $newlistlevel = length($litype);
	  }
	  
	  if ($newlistlevel > 0) {
	      my $listopen;
	      my $listclose;
	      
	      $listart = '<li>';
	      $liend = '</li>';
	      $line = $liitem;
	      if (substr($litype, 0, 1) eq '*') {
		  $listopen = "<ul>";
		  $listclose = "</ul>";
	      } elsif (substr($litype, 0, 1) eq '#') {
		  $listopen = "<ol>";
		  $listclose = "</ol>";
	      } else {
		  $newdata .= "\n\nUnknown list type " . substr($litype, 0, 1) . " - please file a bug report.\n\n";
		  $listopen = "<ul>"; # Use the ul list type
		  $listclose = "</ul>";
	      }
	      if ($newlistlevel < $curlistlevel) {
		  while ($newlistlevel < $curlistlevel) {
		      $newdata .= shift(@closelist) . "\n";
		      $curlistlevel--;
		  }
	      }
	      if ($newlistlevel > $curlistlevel) {
		  while ($newlistlevel > $curlistlevel) {
		      $newdata .= $listopen . "\n";
		      unshift(@closelist, $listclose);
		      $curlistlevel++;
		  }
	      }
	  } elsif ($curlistlevel > 0) {
	      # We are no longer in a list; close out the existing lists
	      while ($curlistlevel > 0) {
		  $newdata .= shift(@closelist) . "\n";
		  $curlistlevel--;
	      }
	  }
	  $newdata .= $listart . $line . $liend . "\n";
      }
      $_ = $newdata;
  }

  # Replace preformatted sections
  s/\~pp\~(\d+?)\~\/pp\~/"<pre>" . $pp[$1] . "<\/pre>"/ge;
  # Replace non-parsed sections
  s/\~np\~(\d+?)\~\/np\~/$np[$1]/ge;

  # Print style definitions for the unique tiki styles.
  # FIXME: This shouldn't be hard-coded, of course.
  print <<ENDSTYLE;
  <style type="text/css">
    <!--
      div.simplebox {  
	margin-left: 1px;  margin-right: 1px;  margin-top: 5px;  margin-bottom: 5px;
	padding-left: 2px;  padding-top: 2px;  padding-bottom: 2px;  padding-right: 2px;
        color: black;
        border: 1px solid #8CACBB;  background: white;  
      }

      div.titlebar {
	background: #C7D0D9;
	color: black;
	border: 1px solid #8CACBB;
	padding-left: 1px; 
	padding-right: 1px; 
	padding-top: 1px;
	padding-bottom: 1px;
	margin: 1px 1px;
	clear: both;
      }
    -->
    </style>
ENDSTYLE

  # make paragraphs of the text
  # and do the post-processing
  my @nodes = split /\n{2,}/;
  my $out = "";
  my $islist = 0;
  my @list = ();
  foreach my $node (@nodes) {
    next if($node =~ /^\s*$/s);
    if ($node =~ /^<h\d>/) {
      # head, feed out unchanged
      print "$node\n";
    }
    elsif ($node =~ /^\s/m) {
      # verbatim
      print "<pre>$node</pre>\n";
    }
    elsif ($node =~ /^(P<[^>]*>)$/s) {
      # include another page
      my $tree = new Pod::Tree;
      $tree->load_string("P<$1>");
      $tree->walk(\&walker);
    }
    else {
      print "<p>$node</p>\n";
    }
  }
}

END {
}

1; #end

__END__

=head1 NAME

PodWiki::Formatter::Tiki - PodWiki formatter that understands (much of) the TikiWiki markup language

=head1 SYNOPSIS

  use PodWiki::Formatter::Tiki;
  
  $fmtr = new Pod::Formatter::Tiki;
  @tags = $fmtr->tags;
  $fmtr->format($text);
  

=head1 REQUIRES

Perl 5.6.0, Exporter, IO::File, Pod::Tree::Node


=head1 EXPORTS

Nothing


=head1 DESCRIPTION

C<PodWiki::Formatter::Tiki> parses text that uses the TikiWiki markup
language and prints the equivalent HTML. This is intended to be used
by PodWiki users that are migrating content from a TikiWiki
system. See L<http://tikiwiki.org/> for a description of the markup
lagnuage. Not all features of TikiWiki are supported, including (but
not limited to):

=over 4

=item Most plugins, which take the form of: {NAME(args)}...{NAME}

=item {maketoc} (Table of contents generator)

=item tables

=item rss feeds

=item dynamic content

=back


=head1 METHODS

=over 4

=item I<$fmtr> = C<new> C<PodWiki::Formatter>

Creates a new C<PodWiki::Formatter> object.

=item I<@tags> = I<$fmtr>->C<tags>()

Returns an array containing the tags that are processed by the formatter.

=item I<$ok> = I<$fmtr>->C<format>(I<$text>)

Parses I<$text> according to the rules of the formatter. The resultant
text, if any, is written to STDOUT.

=back

=head1 DIAGNOSTICS

=over 4

=back


=head1 NOTES

None.

=head1 SEE ALSO

perl(1), L<C<PodWiki::Runner>>


=head1 ACKNOWLEDGMENTS

=over 4

=item *

The TikiWiki Community L<http://tikiwiki.org>

=back


=head1 AUTHOR

David Bright <BrightD@acm.org>


=head1 COPYRIGHT

Copyright 2004 by David Bright. This module is free software; you can
redistribute it and/or modify it under the same terms as Perl itself.
